@@ -24,10 +24,11 @@
 ----------------------------------------------------------------------------
 module Data.Functor.Alt
   ( Alt(..)
+  , optional
   , module Data.Functor.Apply
   ) where
 
-import Control.Applicative hiding (some, many)
+import Control.Applicative hiding (some, many, optional)
 import Control.Applicative.Backwards
 import Control.Applicative.Lift
 import Control.Arrow
@@ -50,8 +51,8 @@ import Data.Functor.Bind
 import Data.Functor.Compose
 import Data.Functor.Product
 import Data.Functor.Reverse
-import Data.Semigroup hiding (Product)
 import Data.List.NonEmpty (NonEmpty(..))
+import Data.Semigroup (Option(..), Semigroup(..))
 import Prelude (($),Either(..),Maybe(..),const,IO,Ord,(++),(.),either,seq,undefined)
 import Unsafe.Coerce
 
@@ -61,6 +62,11 @@ import Data.IntMap (IntMap)
 import Data.Sequence (Seq)
 import qualified Data.Map as Map
 import Data.Map (Map)
+# if MIN_VERSION_base(4,8,0)
+import Prelude (mappend)
+# else
+import Data.Monoid (mappend)
+# endif
 #endif
 
 #if defined(MIN_VERSION_tagged) || (MIN_VERSION_base(4,7,0))
@@ -73,6 +79,7 @@ import Generics.Deriving.Base
 import GHC.Generics
 #endif
 
+
 infixl 3 <!>
 
 -- | Laws:
@@ -117,6 +124,10 @@ class Functor f => Alt f where
     where many_v = some_v <!> pure []
           some_v = (:) <$> v <*> many_v
 
+-- | One or none.
+optional :: (Alt f, Applicative f) => f a -> f (Maybe a)
+optional v = Just <$> v <!> pure Nothing
+
 instance (Alt f, Alt g) => Alt (f :*: g) where
   (as :*: bs) <!> (cs :*: ds) = (as <!> cs) :*: (bs <!> ds)
 
