@@ -80,6 +80,7 @@ import Data.Functor.Reverse
 import Data.Functor.Extend
 import Data.List.NonEmpty
 import Data.Orphans ()
+import Language.Haskell.TH (Q)
 import Prelude hiding (id, (.))
 
 #if MIN_VERSION_base(4,4,0)
@@ -144,6 +145,7 @@ infixl 4 <.>, <., .>
 -- @
 class Functor f => Apply f where
   (<.>) :: f (a -> b) -> f a -> f b
+  (<.>) = liftF2 id
 
   -- | @ a '.>' b = 'const' 'id' '<$>' a '<.>' b @
   (.>) :: f a -> f b -> f b
@@ -153,6 +155,15 @@ class Functor f => Apply f where
   (<.) :: f a -> f b -> f a
   a <. b = const <$> a <.> b
 
+  -- | Lift a binary function into a comonad with zipping
+  liftF2 :: (a -> b -> c) -> f a -> f b -> f c
+  liftF2 f a b = f <$> a <.> b
+  {-# INLINE liftF2 #-}
+
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
+  {-# MINIMAL (<.>) | liftF2 #-}
+#endif
+
 #ifdef MIN_VERSION_tagged
 instance Apply (Tagged a) where
   (<.>) = (<*>)
@@ -269,6 +280,11 @@ instance Apply Complex where
   (a :+ b) <.> (c :+ d) = a c :+ b d
 #endif
 
+-- Applicative Q was only added in template-haskell 2.7 (GHC 7.4), so
+-- define in terms of Monad instead.
+instance Apply Q where
+  (<.>) = ap
+
 #ifdef MIN_VERSION_containers
 -- | A Map is not 'Applicative', but it is an instance of 'Apply'
 instance Ord k => Apply (Map k) where
@@ -497,6 +513,9 @@ instance Bind Option where
 instance Bind Identity where
   (>>-) = (>>=)
 
+instance Bind Q where
+  (>>-) = (>>=)
+
 instance Bind m => Bind (IdentityT m) where
   IdentityT m >>- f = IdentityT (m >>- runIdentityT . f)
 
