@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, TypeOperators #-}
 -----------------------------------------------------------------------------
 -- |
 -- Copyright   :  (C) 2011-2015 Edward Kmett
@@ -39,16 +39,28 @@ import Data.List.NonEmpty (NonEmpty(..))
 import Data.Semigroup
 import Data.Semigroup.Foldable
 import Data.Semigroup.Bifoldable
+#ifdef MIN_VERSION_tagged
 import Data.Tagged
+#endif
 #if __GLASGOW_HASKELL__ < 710
 import Data.Traversable
 #endif
 import Data.Traversable.Instances ()
 
+#if MIN_VERSION_base(4,4,0)
+import Data.Complex
+#endif
+
 #ifdef MIN_VERSION_containers
 import Data.Tree
 #endif
 
+#ifdef MIN_VERSION_generic_deriving
+import Generics.Deriving.Base
+#else
+import GHC.Generics
+#endif
+
 class (Bifoldable1 t, Bitraversable t) => Bitraversable1 t where
   bitraverse1 :: Apply f => (a -> f b) -> (c -> f d) -> t a c -> f (t b d)
   bitraverse1 f g  = bisequence1 . bimap f g
@@ -92,9 +104,11 @@ instance Bitraversable1 Const where
   bitraverse1 f _ (Const a) = Const <$> f a
   {-# INLINE bitraverse1 #-}
 
+#ifdef MIN_VERSION_tagged
 instance Bitraversable1 Tagged where
   bitraverse1 _ g (Tagged b) = Tagged <$> g b
   {-# INLINE bitraverse1 #-}
+#endif
 
 instance (Bitraversable1 p, Traversable1 f, Traversable1 g) => Bitraversable1 (Biff p f g) where
   bitraverse1 f g = fmap Biff . bitraverse1 (traverse1 f) (traverse1 g) . runBiff
@@ -130,6 +144,7 @@ instance Bitraversable1 p => Bitraversable1 (WrappedBifunctor p) where
   bitraverse1 f g = fmap WrapBifunctor . bitraverse1 f g . unwrapBifunctor
   {-# INLINE bitraverse1 #-}
 
+
 class (Foldable1 t, Traversable t) => Traversable1 t where
   traverse1 :: Apply f => (a -> f b) -> t a -> f (t b)
   sequence1 :: Apply f => t (f b) -> f (t b)
@@ -141,6 +156,28 @@ class (Foldable1 t, Traversable t) => Traversable1 t where
   {-# MINIMAL traverse1 | sequence1 #-}
 #endif
 
+instance Traversable1 f => Traversable1 (Rec1 f) where
+  traverse1 f (Rec1 as) = Rec1 <$> traverse1 f as
+
+instance Traversable1 f => Traversable1 (M1 i c f) where
+  traverse1 f (M1 as) = M1 <$> traverse1 f as
+
+instance Traversable1 Par1 where
+  traverse1 f (Par1 a) = Par1 <$> f a
+
+instance Traversable1 V1 where
+  traverse1 _ v = v `seq` undefined
+
+instance (Traversable1 f, Traversable1 g) => Traversable1 (f :*: g) where
+  traverse1 f (as :*: bs) = (:*:) <$> traverse1 f as <.> traverse1 f bs
+
+instance (Traversable1 f, Traversable1 g) => Traversable1 (f :+: g) where
+  traverse1 f (L1 as) = L1 <$> traverse1 f as
+  traverse1 f (R1 bs) = R1 <$> traverse1 f bs
+
+instance (Traversable1 f, Traversable1 g) => Traversable1 (f :.: g) where
+  traverse1 f (Comp1 m) = Comp1 <$> traverse1 (traverse1 f) m
+
 instance Traversable1 Identity where
   traverse1 f = fmap Identity . f . runIdentity
 
@@ -167,6 +204,17 @@ instance (Traversable1 f, Traversable1 g) => Traversable1 (Functor.Sum f g) wher
   traverse1 f (Functor.InL x) = Functor.InL <$> traverse1 f x
   traverse1 f (Functor.InR y) = Functor.InR <$> traverse1 f y
 
+#if MIN_VERSION_base(4,4,0)
+instance Traversable1 Complex where
+  traverse1 f (a :+ b) = (:+) <$> f a <.> f b
+  {-# INLINE traverse1 #-}
+#endif
+
+#ifdef MIN_VERSION_tagged
+instance Traversable1 (Tagged a) where
+  traverse1 f (Tagged a) = Tagged <$> f a
+#endif
+
 #ifdef MIN_VERSION_containers
 instance Traversable1 Tree where
   traverse1 f (Node a []) = (`Node`[]) <$> f a
