@@ -1,4 +1,5 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE GADTs #-}
 
 #if __GLASGOW_HASKELL__ >= 706
 {-# LANGUAGE PolyKinds #-}
@@ -47,6 +48,11 @@ import Control.Comonad
 import Data.Tagged (Tagged (..))
 #endif
 
+#if MIN_VERSION_base(4,7,0)
+import qualified Data.Type.Coercion as Co
+import qualified Data.Type.Equality as Eq
+#endif
+
 -- | 'Control.Category.Category' sans 'Control.Category.id'
 class Semigroupoid c where
   o :: c j k -> c i j -> c i k
@@ -96,3 +102,16 @@ instance Semigroupoid Const where
 instance Semigroupoid Tagged where
   Tagged b `o` _ = Tagged b
 #endif
+
+#if MIN_VERSION_base(4,7,0)
+instance Semigroupoid Co.Coercion where
+  o = flip Co.trans
+
+instance Semigroupoid (Eq.:~:) where
+  o = flip Eq.trans
+#endif
+
+#if MIN_VERSION_base(4,10,0)
+instance Semigroupoid (Eq.:~~:) where
+  o Eq.HRefl Eq.HRefl = Eq.HRefl
+#endif
